module CrmJavaSearch
  module Checkers
    class BaseChecker

      def initialize(user:, entity:, params:)
        params = params.with_indifferent_access
        Log.logger.info("crm_java_search es initialize params  #{params.to_json}")

        @user         = user
        @entity       = entity
        @organization = user.organization
        @field        = params["field"]
        @field_value  = params["field_value"]
        @exclude_ids  = params["exclude_ids"].compact || []
        @exclude_ids.push(@entity.id) if @entity && @entity.id
        @id_source = params["id_source"] || ''
        @custom_field_template_id = params["custom_field_template_id"]
        @no_custom_field_template = -1
      end

      def es_check(klass)
        @duplicated = false
        @model_human_name  = @organization.feature_label(klass.name)
        @custom_field_name = @organization.send("#{klass.name.underscore}_duplicate_field_custom_field_map")[@field]
        @custom_field      = @organization.send("#{klass.name.underscore}_duplicate_custom_fields").
          find{ |custom_field|custom_field.name == @custom_field_name }

        return field_not_found if @custom_field.blank?
        
        @field_label = @custom_field.label
        @cross_fields = get_cross_fields

        unless (@organization.duplicate_check_enabled?(key: klass.name.underscore) &&
          @organization.send("should_#{klass.name.underscore}_check_duplicate_for?", @field))
          return field_check_disabled
        end

        check_service(klass)
      end

      def check_service(klass)
        es_query = {@custom_field.name => @field_value, "exclude_ids" =>  @exclude_ids, "cross_fields" => @cross_fields, "id_source" => @id_source}
        Log.logger.info("crm_java_search es request params es_query: #{es_query} ")
        @res = CrmJavaSearch::Agents.const_get("#{klass.name}Agent").repeat_check(klass.name.underscore, @organization.id, es_query)
        Log.logger.info("crm_java_search es response #{@res}")
        return unknown_error unless [0, 500, 502].include? @res['code'].to_i
        return system_error if @res['code'].to_i == 500
        return parameter_error if @res['code'].to_i == 502

        if @res["data"]['repeat'] && @res["data"]['entities'].present?
          @res["data"]['entities'].select{ |entity_hash|
            Log.logger.info("crm_java_search es entity_hash is  #{entity_hash.to_json}")
  
            if klass.name == 'Customer' && @custom_field_template_id.present? && @organization.customers.find(entity_hash['id'].to_i).custom_field_template_id == @custom_field_template_id.to_i && check_by_custom_field_template
              Log.logger.info("crm_java_search es 客户业务模板内查重")
  
              #TODO 客户业务模板内查重
              check_in_custom_field_template
            elsif klass.name == 'Customer' && @organization.customers.find(entity_hash['id'].to_i).custom_field_template_id.blank? && check_by_custom_field_template
              Log.logger.info("crm_java_search es 客户无业务模板，查重以前数据")
  
              #TODO 客户无业务模板，查重以前数据
              check_in_no_custom_field_template
            elsif klass.name == 'Contact' && @custom_field_template_id.present? && @organization.contacts.find(entity_hash['id'].to_i).customer.custom_field_template_id == @custom_field_template_id.to_i && check_by_custom_field_template
              Log.logger.info("crm_java_search es 联系人业务模板内查重")
  
              #TODO 联系人业务模板内查重
              check_in_custom_field_template
            elsif klass.name == 'Contact' && @organization.contacts.find(entity_hash['id'].to_i).customer.custom_field_template_id.blank? && check_by_custom_field_template
              Log.logger.info("crm_java_search es 联系人无业务模板，查重以前数据")
  
              #TODO 联系人无业务模板，查重以前数据
              check_in_no_custom_field_template
            elsif (@organization.settings(:customer_duplicate_scope_config).type == 'all') || (%w(Contact Customer).exclude? klass.name)
              duplicate_in_owns_or_org(entity_hash)
            else
              no_duplicates
            end
          }
        else
          no_duplicates
        end
      end

      def check_in_custom_field_template
        if @organization.settings(:customer_duplicate_scope_config).custom_template_scopes.include? @custom_field_template_id.to_s
          return duplicate_in_custom_template
        else
          return no_duplicate_in_custom_template
        end
      end

      def check_in_no_custom_field_template
        if @organization.settings(:customer_duplicate_scope_config).custom_template_scopes.include? @no_custom_field_template.to_s
          return duplicate_in_no_custom_template
        else
          return no_duplicate_in_no_custom_template
        end
      end

      def check_by_custom_field_template
        !!(@organization.settings(:customer_duplicate_scope_config).type == 'by_custom_template')
      end

      def duplicate_in_owns_or_org(entity_hash)
        return duplicate_in_owns(entity_hash['cross_fields']) if entity_hash['user_id'].to_i == @user.id
        duplicate_in_org(entity_hash['cross_fields'])
      end

      def cross_field_from_setting
        RelationDuplicate::RelationService.new(@organization).send("#{@entity.class.name.underscore}_cross")[@custom_field.name]
      end
      
      def get_cross_fields
        base_field = {@entity.class.name.underscore => @custom_field.name}
        if cross_field_from_setting
          cross_field_from_setting.merge(base_field).with_indifferent_access
        else
          base_field.with_indifferent_access
        end
      end

      def field_check_disabled
        @code = "0"
        @remark = "#{@model_human_name}模块下#{@field_label}字段未开启查重"
      end

      def field_not_found
        @code = "0"
        @remark = "#{@model_human_name}模块下未找到查重字段#{@field}对应的业务字段"
      end

      def no_duplicates
        @code = "0"
        @remark = "未发现相同#{@field_label}的#{@model_human_name}"
      end

      def duplicate_in_no_custom_template
        @code = "1"
        @remark = "你已经有相同#{@field_label}的#{@model_human_name}在无业务模板里面"
        @duplicated = true
      end

      def no_duplicate_in_no_custom_template
        @code = "0"
        @remark = "未发现相同#{@field_label}的#{@model_human_name}在无业务模板里面"
        @duplicated = false
      end

      def no_duplicate_in_custom_template
        @code = "0"
        @remark = "未发现相同#{@field_label}的#{@model_human_name}在业务模板里面"
        @duplicated = false
      end

      def duplicate_in_custom_template
        @code = "1"
        @remark = "你已经有相同#{@field_label}的#{@model_human_name}在业务模板里面"
        @duplicated = true
      end

      def duplicate_in_owns(cross_field=nil)
        @code = "1"
        if cross_field.present?
          @remark = "你已经有" + cross_fields_duplicate(cross_field)
        else
          @remark = "你已经有相同#{@field_label}的#{@model_human_name}"
          @duplicated = true
        end
      end

      def duplicate_in_org(cross_field=nil)
        @code = "2"
        if cross_field.present?
          @remark = "公司内已有" + cross_fields_duplicate(cross_field)
        else
          @remark = "公司内已有相同#{@field_label}的#{@model_human_name}"
          @duplicated = true
        end
      end

      def system_error
        @duplicated = true
        @code = "3"
        @remark = "#{@model_human_name}查重系统异常"
      end

      def parameter_error
        @duplicated = true
        @code = "4"
        @remark = "#{@model_human_name}查重参数错误"
      end

      def unknown_error
        @duplicated = true
        @code = "5"
        @remark = "#{@model_human_name}查重为定义错误(code!=0)"
      end

      def cross_fields_duplicate(cross_field)
        @duplicated = true
        labels = if @entity.present? && cross_field.keys.include?(@entity.class.name.underscore)
                   { 'klass_to_show' => @entity.class.name,
                     'klass_model_label' => @organization.feature_label(@entity.class.name) }
                 else
                   { 'klass_to_show' => cross_field.keys.first,
                     'klass_model_label' => @organization.feature_label(cross_field.keys.first.classify) }
                 end

        if labels['klass_to_show'] == 'customer' && @custom_field_name == 'company_name'
          customer_field_label = @organization.customer_duplicate_custom_fields.detect{ |c| c.name == 'name' }.try(:label)
          "相同#{ customer_field_label }的#{ labels['klass_model_label'] }"
        elsif labels['klass_to_show'] == 'lead' && @custom_field_name == 'name'
          lead_field_label = @organization.lead_duplicate_custom_fields.detect{ |c| c.name == 'company_name'}.try(:label)
          "相同#{ lead_field_label }的#{ labels['klass_model_label'] }"
        else
          "相同#{ @field_label }的#{ labels['klass_model_label'] }"
        end
      end

      def duplicated?
        !!@duplicated
      end

      def result
        { code: @code, remark: @remark }
      end

      def remark
        @remark
      end

    end
  end
end
