module CrmJavaSearch
  module RelationDuplicate
    class BaseService
      attr_reader :organization
      attr_accessor :cross
      # 子类必须实现duplicate_field_config 方法
      # 子类名称必须是实体类名加Service
      def initialize(organization)
        @organization = organization
        @cross = false
      end

      def system_duplicate_fields
        enable_system_custom_fields = system_custom_fields.select do |column|
          system_duplicate_fields_search.include?(column[:name])
        end
        enable_system_custom_fields.map do |column|
          confirm_duplicate_field(column)
        end
      end

      def custom_duplicate_fields
        organization.allow_duplicate_user_custom_fields(entity_model_klass).map do |column|
          confirm_duplicate_field(column)
        end
      end

      def duplicate_fields
        system_duplicate_fields + custom_duplicate_fields
      end

      def cross_fields
        duplicate_fields.map do |field|
          if field[:checked]
            field.delete(:checked)
            field
          end
        end.compact
      end

      private

      def confirm_duplicate_field(column)
        if cross
          cross_duplicate_fields_hash(column)
        else
          duplicate_fields_hash(column)
        end
      end

      def duplicate_fields_hash(column)
        name = column[:name].start_with?('address.') ? column[:name].split('.').last : column[:name]
        { name: name, label: column[:label], checked: duplicate_field_config.send(name).eql?('1') }
      end

      def cross_duplicate_fields_hash(column)
        name = column[:name].start_with?('address.') ? column[:name].split('.').last : column[:name]
        { name: column[:name], label: column[:label], checked: duplicate_field_config.send(name).eql?('1') }
      end

      def system_custom_fields
        @system_custom_fields ||= entity_model_klass.cache_enabled_custom_fields_by_organization(organization.id)
      end

      def entity_model_klass
        self.class.name.demodulize.gsub('Service', '').constantize
      end
    end
  end
end
